functor MLJLexFun(structure Tokens:MLJ_TOKENS)=
   struct
    structure UserDeclarations =
      struct
(*......................................................................*)
(* MLj - a Standard ML to Java bytecode compiler                        *)
(* Copyright (C) 1999 Persimmon IT Inc.                                 *)
(*                                                                      *)
(* This program is free software; you can redistribute it and/or        *)
(* modify it under the terms of the GNU General Public License          *)
(* as published by the Free Software Foundation; either version 2       *)
(* of the License, or (at your option) any later version.               *)
(*                                                                      *)
(* This program is distributed in the hope that it will be useful,      *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of       *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *)
(* GNU General Public License for more details.                         *)
(*                                                                      *)
(* You should have received a copy of the GNU General Public License    *)
(* along with this program; if not, write to the Free Software          *)
(* Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *)
(* MA 02111-1307, USA.                                                  *)
(*......................................................................*)

(* NOTE ON COMPATIBILITY.
   The only documented way of getting at strings in ML-Lex is to use the
   yytext variable.  However this requires allocating a new string and
   copying it, which is undesirable.  So we use the following ghastly
   hack.  The way ML-Lex works, each action has access to the following
   variables:

      Variable  Type   
      yyb       string ref
      i0        index in !yyb of the start of the token
      i         index in !yyb of the character after the end of the token.
   
   Thus there is usually no need to copy.
   ALL calls to functions TokenTable.lookup_XXXX take an argument
   which will be exactly of the following form:
      "{characters= !yyb,initial=i0,length=i-i0,position=yypos}"

   We also use the same trick to get at normal characters of strings.
   For this we use code exactly of the form
      "String.sub(!yyb,i0)"
 

   If the names or meanings of the variables yyb, i0, i change, search for 
   these strings and modify the code.  (Searching for "yyb" should find
   them all anyway).  It would be nice if
   ML-Lex would be changed to give access to variables (say) yybuffer,
   yytextstart, yytextend instead. 

   We still use yytext in a number of cases.
   The most time-consuming are probably numerical constants.
   These get passed down as strings anyway.  (Since we don't
   know the type, the alternative would be to pass them as
   IntInfs which might actually be slower.)  We could save on creating
   yytext for all integer and word constants which aren't just a 
   sequence of digits, but it doesn't seem worthwhile.
   The other main use is in string numerical escapes (\ddd or \udddd)
   which I hope are comparatively rare.

   Making lexing faster still:
      1) There are a lot of integer additions of yypos.  These presumably
         require costly overflow checks on some architectures.
         (For example, Alphas do have an add-with-overflow check but
         presumably SML/NJ has to insert trap boundaries so that the
         overflows get raised in the right place).  An obvious
         solution is to use word addition instead.
      2) Find some way to do string constants which doesn't involve
         creating a list of characters.  EG one could construct
         a regular expression for a complete string constant.
         But this would involve parsing the string twice and also
         degrade errors correction, so I doubt if it's worth it.
         
   *)


(* User definitions section *)

(* The next 4 lines are copied straight from the ML-Yacc manual; I don't 
   understand them all
   *)
type pos = Position.pos
type svalue = Tokens.svalue
type ('a,'b) token = ('a,'b) Tokens.token
type lexresult = (svalue,pos) token


val size=String.size

open Tokens
open LexState

structure TokenTable=TokenTable(Tokens)

fun cors()=
(case !string_type of
   LexState.CHAR => "character"
|  LexState.STRING => "string"
|  LexState.JAVASYMBOL => "backquoted Java symbol"
)

structure ICF=IntConvFlags

val bad_token=BAD(~1,~1) (* This is used in the HELDOVER state *)

val bad_symbol=Symbol.symbolAsciiString("????")

val held_over = ref bad_token 

(* End-of-file function *)
fun eof()=
let
   val eofpos=SourceMap.eof(!sourcemap)

   val ()=
      if !LexState.comment_level<>0
      then
         LexState.err(Error.error({left= !LexState.comment_start,right=eofpos},
            "EOF in comment"))
      else
         ()

   val ()=
      if !LexState.string_start>=0
      then
         LexState.err(Error.error({left= !LexState.string_start,right=eofpos},"Unclosed string"))
      else
         ()
in
   Tokens.EOF(eofpos,eofpos)
end

fun do_newline yypos=(SourceMap.newline(!LexState.sourcemap,yypos))

fun atoi s=
  (case Int.fromString s of
     SOME i => i
   )

fun atow s=
  (case Word.fromString s of
     SOME w => w
   )

fun end_string yypos=
let
   val st= !string_type
   val token=
      if st=LexState.CHAR
      then
      let
        val w=
          (case !string_contents of
             [w] => w
          |  [] =>
            (err(Error.error({left= !string_start,right=yypos+1},
                "Empty character constant"));
             0wxff)
          |  w::_ =>
            (err(Error.error({left= !string_start,right=yypos+1},
                "Character constant has more than 1 character"));
             w)
         )
         val ji=JavaInt.fromInt(Word.toInt w)
         val scon=SCon.CharCon ji
      in
         SCON(scon,!string_start,yypos+1)
      end
      else
      let
         val words=List.rev(!string_contents)
         val js=JavaString.fromUnicode words
      in
         if st=LexState.STRING
         then
           (if JavaString.containsAscii(js,#".")
            then
                JAVALONGID
            else
                JAVAID)
               (js,!string_start,yypos+1)
         else
            ALPHA(Symbol.symbol js,!string_start,yypos+1)
      end
   val ()=(string_start:= ~1;string_contents:= [];string_type:=NOT)
in
   token
end

fun suffix(s,n)=String.extract(s,n,NONE)   


(* definitions section.  
   ml-lex is objecting to be putting comments in it so here they are.
   ws1 is space/tab/form-feed.
   An id is something to be looked up with TokenTable.lookup_id.
   A tyvar_id is something to be looked up with TokenTable.lookup_tyvar.
   An mlj_id is something to be looked up with TokenTable.lookup_underline.
   These MUST ALL be a subset of _{alpha_id}.
   We need a special token for integers not beginning with 0 because
   they can also be record labels.
   *)

end (* end of user routines *)
exception LexError (* raised if illegal leaf action tried *)
structure Internal =
	struct

datatype yyfinstate = N of int
type statedata = {fin : yyfinstate list, trans: string}
(* transition & final state table *)
val tab = let
val s = [ 
 (0, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000"
),
 (1, 
"\013\013\013\013\013\013\013\013\013\062\065\013\062\064\013\013\
\\013\013\013\013\013\013\013\013\013\013\013\013\013\013\013\013\
\\062\030\061\059\030\030\030\057\055\054\052\030\051\030\046\030\
\\038\036\036\036\036\036\036\036\036\036\030\035\030\030\030\030\
\\030\028\028\028\028\028\028\028\028\028\028\028\028\028\028\028\
\\028\028\028\028\028\028\028\028\028\028\028\034\030\033\030\031\
\\030\028\028\028\028\028\028\028\028\028\028\028\028\028\028\028\
\\028\028\028\028\028\028\028\028\028\028\028\027\026\025\014\013\
\\013"
),
 (3, 
"\000\000\000\000\000\000\000\000\000\066\000\000\066\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\066\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000"
),
 (5, 
"\067\067\067\067\067\067\067\067\067\067\065\067\067\072\067\067\
\\067\067\067\067\067\067\067\067\067\067\067\067\067\067\067\067\
\\067\067\067\067\067\067\067\067\070\067\068\067\067\067\067\067\
\\067\067\067\067\067\067\067\067\067\067\067\067\067\067\067\067\
\\067\067\067\067\067\067\067\067\067\067\067\067\067\067\067\067\
\\067\067\067\067\067\067\067\067\067\067\067\067\067\067\067\067\
\\067\067\067\067\067\067\067\067\067\067\067\067\067\067\067\067\
\\067\067\067\067\067\067\067\067\067\067\067\067\067\067\067\067\
\\067"
),
 (7, 
"\073\073\073\073\073\073\073\073\073\073\103\073\073\102\073\073\
\\073\073\073\073\073\073\073\073\073\073\073\073\073\073\073\073\
\\074\074\101\074\074\074\074\074\074\074\074\074\074\074\074\074\
\\074\074\074\074\074\074\074\074\074\074\074\074\074\074\074\074\
\\074\074\074\074\074\074\074\074\074\074\074\074\074\074\074\074\
\\074\074\074\074\074\074\074\074\074\074\074\074\076\074\074\074\
\\075\074\074\074\074\074\074\074\074\074\074\074\074\074\074\074\
\\074\074\074\074\074\074\074\074\074\074\074\074\074\074\074\073\
\\073"
),
 (9, 
"\104\104\104\104\104\104\104\104\104\106\065\104\106\107\104\104\
\\104\104\104\104\104\104\104\104\104\104\104\104\104\104\104\104\
\\106\104\104\104\104\104\104\104\104\104\104\104\104\104\104\104\
\\104\104\104\104\104\104\104\104\104\104\104\104\104\104\104\104\
\\104\104\104\104\104\104\104\104\104\104\104\104\104\104\104\104\
\\104\104\104\104\104\104\104\104\104\104\104\104\105\104\104\104\
\\104\104\104\104\104\104\104\104\104\104\104\104\104\104\104\104\
\\104\104\104\104\104\104\104\104\104\104\104\104\104\104\104\104\
\\104"
),
 (11, 
"\108\108\108\108\108\108\108\108\108\114\065\108\114\115\108\108\
\\108\108\108\108\108\108\108\108\108\108\108\108\108\108\108\108\
\\114\109\108\109\109\109\109\111\108\108\109\109\108\109\108\109\
\\111\111\111\111\111\111\111\111\111\111\109\108\109\109\109\109\
\\109\111\111\111\111\111\111\111\111\111\111\111\111\111\111\111\
\\111\111\111\111\111\111\111\111\111\111\111\108\109\108\109\111\
\\113\111\111\111\111\111\111\111\111\111\111\111\111\111\111\111\
\\111\111\111\111\111\111\111\111\111\111\111\108\109\108\109\108\
\\108"
),
 (14, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\015\000\015\015\015\015\000\000\000\015\015\000\015\000\015\
\\022\016\016\016\016\016\016\016\016\016\015\000\015\015\015\015\
\\015\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\015\000\015\000\
\\015\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\015\000\015\000\
\\000"
),
 (15, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\015\000\015\015\015\015\000\000\000\015\015\000\015\000\015\
\\000\000\000\000\000\000\000\000\000\000\015\000\015\015\015\015\
\\015\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\015\000\015\000\
\\015\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\015\000\015\000\
\\000"
),
 (16, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\020\000\
\\016\016\016\016\016\016\016\016\016\016\000\000\000\000\000\000\
\\000\000\000\000\000\017\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\017\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000"
),
 (17, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\019\019\019\019\019\019\019\019\019\019\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\018\000\
\\000"
),
 (18, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\019\019\019\019\019\019\019\019\019\019\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000"
),
 (20, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\021\021\021\021\021\021\021\021\021\021\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000"
),
 (21, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\021\021\021\021\021\021\021\021\021\021\000\000\000\000\000\000\
\\000\000\000\000\000\017\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\017\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000"
),
 (22, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\020\000\
\\016\016\016\016\016\016\016\016\016\016\000\000\000\000\000\000\
\\000\000\000\000\000\017\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\017\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\023\000\000\000\000\000\000\000\
\\000"
),
 (23, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\024\024\024\024\024\024\024\024\024\024\000\000\000\000\000\000\
\\000\024\024\024\024\024\024\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\024\024\024\024\024\024\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000"
),
 (28, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\029\000\000\000\000\000\000\000\000\
\\029\029\029\029\029\029\029\029\029\029\000\000\000\000\000\000\
\\000\029\029\029\029\029\029\029\029\029\029\029\029\029\029\029\
\\029\029\029\029\029\029\029\029\029\029\029\000\000\000\000\029\
\\000\029\029\029\029\029\029\029\029\029\029\029\029\029\029\029\
\\029\029\029\029\029\029\029\029\029\029\029\000\000\000\000\000\
\\000"
),
 (31, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\032\032\032\032\032\032\032\032\032\032\032\032\032\032\032\
\\032\032\032\032\032\032\032\032\032\032\032\000\000\000\000\000\
\\000\032\032\032\032\032\032\032\032\032\032\032\032\032\032\032\
\\032\032\032\032\032\032\032\032\032\032\032\000\000\000\000\000\
\\000"
),
 (32, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\032\000\000\000\000\000\000\000\000\
\\032\032\032\032\032\032\032\032\032\032\000\000\000\000\000\000\
\\000\032\032\032\032\032\032\032\032\032\032\032\032\032\032\032\
\\032\032\032\032\032\032\032\032\032\032\032\000\000\000\000\032\
\\000\032\032\032\032\032\032\032\032\032\032\032\032\032\032\032\
\\032\032\032\032\032\032\032\032\032\032\032\000\000\000\000\000\
\\000"
),
 (36, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\020\000\
\\037\037\037\037\037\037\037\037\037\037\000\000\000\000\000\000\
\\000\000\000\000\000\017\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\017\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000"
),
 (38, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\020\000\
\\045\045\045\045\045\045\045\045\045\045\000\000\000\000\000\000\
\\000\000\000\000\000\017\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\017\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\041\039\000\000\000\000\000\000\000\
\\000"
),
 (39, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\040\040\040\040\040\040\040\040\040\040\000\000\000\000\000\000\
\\000\040\040\040\040\040\040\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\040\040\040\040\040\040\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000"
),
 (41, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\044\044\044\044\044\044\044\044\044\044\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\042\000\000\000\000\000\000\000\
\\000"
),
 (42, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\043\043\043\043\043\043\043\043\043\043\000\000\000\000\000\000\
\\000\043\043\043\043\043\043\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\043\043\043\043\043\043\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000"
),
 (44, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\044\044\044\044\044\044\044\044\044\044\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000"
),
 (45, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\020\000\
\\045\045\045\045\045\045\045\045\045\045\000\000\000\000\000\000\
\\000\000\000\000\000\017\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\017\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000"
),
 (46, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\049\000\000\000\000\000\000\000\000\000\000\047\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000"
),
 (47, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\048\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000"
),
 (49, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\050\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000"
),
 (52, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\015\000\015\015\015\015\000\000\053\015\015\000\015\000\015\
\\000\000\000\000\000\000\000\000\000\000\015\000\015\015\015\015\
\\015\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\015\000\015\000\
\\015\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\015\000\015\000\
\\000"
),
 (55, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\056\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000"
),
 (57, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\058\000\000\000\000\000\000\000\000\
\\058\058\058\058\058\058\058\058\058\058\000\000\000\000\000\000\
\\000\058\058\058\058\058\058\058\058\058\058\058\058\058\058\058\
\\058\058\058\058\058\058\058\058\058\058\058\000\000\000\000\058\
\\000\058\058\058\058\058\058\058\058\058\058\058\058\058\058\058\
\\058\058\058\058\058\058\058\058\058\058\058\000\000\000\000\000\
\\000"
),
 (59, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\015\060\015\015\015\015\000\000\000\015\015\000\015\000\015\
\\000\000\000\000\000\000\000\000\000\000\015\000\015\015\015\015\
\\015\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\015\000\015\000\
\\015\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\015\000\015\000\
\\000"
),
 (62, 
"\000\000\000\000\000\000\000\000\000\063\000\000\063\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\063\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000"
),
 (64, 
"\000\000\000\000\000\000\000\000\000\000\065\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000"
),
 (68, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\069\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000"
),
 (70, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\071\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000"
),
 (76, 
"\077\077\077\077\077\077\077\077\077\098\100\077\098\099\077\077\
\\077\077\077\077\077\077\077\077\077\077\077\077\077\077\077\077\
\\098\077\097\077\077\077\077\096\077\077\077\077\077\077\077\077\
\\093\093\093\093\093\093\093\093\093\093\077\077\077\077\077\077\
\\077\077\077\077\077\077\077\077\077\077\077\077\077\077\077\077\
\\077\077\077\077\077\077\077\077\077\077\077\077\092\077\090\077\
\\077\089\088\077\077\077\087\077\077\077\077\077\077\077\086\077\
\\077\077\085\077\084\079\078\077\077\077\077\077\077\077\077\077\
\\077"
),
 (79, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\080\080\080\080\080\080\080\080\080\080\000\000\000\000\000\000\
\\000\080\080\080\080\080\080\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\080\080\080\080\080\080\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000"
),
 (80, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\081\081\081\081\081\081\081\081\081\081\000\000\000\000\000\000\
\\000\081\081\081\081\081\081\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\081\081\081\081\081\081\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000"
),
 (81, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\082\082\082\082\082\082\082\082\082\082\000\000\000\000\000\000\
\\000\082\082\082\082\082\082\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\082\082\082\082\082\082\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000"
),
 (82, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\083\083\083\083\083\083\083\083\083\083\000\000\000\000\000\000\
\\000\083\083\083\083\083\083\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\083\083\083\083\083\083\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000"
),
 (90, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\091\091\091\091\091\091\091\091\091\091\091\091\091\091\091\091\
\\091\091\091\091\091\091\091\091\091\091\091\091\091\091\091\091\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000"
),
 (93, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\094\094\094\094\094\094\094\094\094\094\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000"
),
 (94, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\095\095\095\095\095\095\095\095\095\095\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000"
),
 (99, 
"\000\000\000\000\000\000\000\000\000\000\100\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000"
),
 (102, 
"\000\000\000\000\000\000\000\000\000\000\103\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000"
),
 (109, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\110\000\110\110\110\110\000\000\000\110\110\000\110\000\110\
\\000\000\000\000\000\000\000\000\000\000\110\000\110\110\110\110\
\\110\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\110\000\110\000\
\\110\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\110\000\110\000\
\\000"
),
 (111, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\112\000\000\000\000\000\000\000\000\
\\112\112\112\112\112\112\112\112\112\112\000\000\000\000\000\000\
\\000\112\112\112\112\112\112\112\112\112\112\112\112\112\112\112\
\\112\112\112\112\112\112\112\112\112\112\112\000\000\000\000\112\
\\000\112\112\112\112\112\112\112\112\112\112\112\112\112\112\112\
\\112\112\112\112\112\112\112\112\112\112\112\000\000\000\000\000\
\\000"
),
(0, "")]
fun f x = x 
val s = map f (rev (tl (rev s))) 
exception LexHackingError 
fun look ((j,x)::r, i) = if i = j then x else look(r, i) 
  | look ([], i) = raise LexHackingError
fun g {fin=x, trans=i} = {fin=x, trans=look(s,i)} 
in Vector.fromList(map g 
[{fin = [], trans = 0},
{fin = [(N 7)], trans = 1},
{fin = [(N 7)], trans = 1},
{fin = [(N 58)], trans = 3},
{fin = [(N 58)], trans = 3},
{fin = [], trans = 5},
{fin = [], trans = 5},
{fin = [], trans = 7},
{fin = [], trans = 7},
{fin = [(N 7)], trans = 9},
{fin = [(N 7)], trans = 9},
{fin = [(N 7)], trans = 11},
{fin = [(N 7)], trans = 11},
{fin = [(N 271)], trans = 0},
{fin = [(N 18),(N 271)], trans = 14},
{fin = [(N 18)], trans = 15},
{fin = [(N 73),(N 133)], trans = 16},
{fin = [], trans = 17},
{fin = [], trans = 18},
{fin = [(N 133)], trans = 18},
{fin = [], trans = 20},
{fin = [(N 133)], trans = 21},
{fin = [(N 73),(N 133)], trans = 22},
{fin = [], trans = 23},
{fin = [(N 96)], trans = 23},
{fin = [(N 36),(N 271)], trans = 0},
{fin = [(N 18),(N 48),(N 271)], trans = 15},
{fin = [(N 34),(N 271)], trans = 0},
{fin = [(N 13),(N 271)], trans = 28},
{fin = [(N 13)], trans = 28},
{fin = [(N 18),(N 271)], trans = 15},
{fin = [(N 46),(N 271)], trans = 31},
{fin = [(N 55)], trans = 32},
{fin = [(N 32),(N 271)], trans = 0},
{fin = [(N 30),(N 271)], trans = 0},
{fin = [(N 40),(N 271)], trans = 0},
{fin = [(N 62),(N 67),(N 133),(N 271)], trans = 36},
{fin = [(N 62),(N 67),(N 133)], trans = 36},
{fin = [(N 67),(N 133),(N 271)], trans = 38},
{fin = [], trans = 39},
{fin = [(N 84)], trans = 39},
{fin = [], trans = 41},
{fin = [], trans = 42},
{fin = [(N 115)], trans = 42},
{fin = [(N 103)], trans = 44},
{fin = [(N 67),(N 133)], trans = 45},
{fin = [(N 142),(N 271)], trans = 46},
{fin = [], trans = 47},
{fin = [(N 44)], trans = 0},
{fin = [(N 140)], trans = 49},
{fin = [(N 137)], trans = 0},
{fin = [(N 38),(N 271)], trans = 0},
{fin = [(N 18),(N 271)], trans = 52},
{fin = [(N 173)], trans = 0},
{fin = [(N 28),(N 271)], trans = 0},
{fin = [(N 26),(N 271)], trans = 55},
{fin = [(N 162)], trans = 0},
{fin = [(N 24),(N 271)], trans = 57},
{fin = [(N 24)], trans = 57},
{fin = [(N 18),(N 271)], trans = 59},
{fin = [(N 178)], trans = 0},
{fin = [(N 175),(N 271)], trans = 0},
{fin = [(N 7),(N 271)], trans = 62},
{fin = [(N 7)], trans = 62},
{fin = [(N 4),(N 271)], trans = 64},
{fin = [(N 4)], trans = 0},
{fin = [(N 58)], trans = 3},
{fin = [(N 170)], trans = 0},
{fin = [(N 170)], trans = 68},
{fin = [(N 168)], trans = 0},
{fin = [(N 170)], trans = 70},
{fin = [(N 165)], trans = 0},
{fin = [(N 4),(N 170)], trans = 64},
{fin = [(N 269)], trans = 0},
{fin = [(N 262),(N 269)], trans = 0},
{fin = [(N 260),(N 262),(N 269)], trans = 0},
{fin = [(N 262),(N 269)], trans = 76},
{fin = [(N 256)], trans = 0},
{fin = [(N 193),(N 256)], trans = 0},
{fin = [(N 256)], trans = 79},
{fin = [], trans = 80},
{fin = [], trans = 81},
{fin = [], trans = 82},
{fin = [(N 230)], trans = 0},
{fin = [(N 187),(N 256)], trans = 0},
{fin = [(N 199),(N 256)], trans = 0},
{fin = [(N 190),(N 256)], trans = 0},
{fin = [(N 196),(N 256)], trans = 0},
{fin = [(N 184),(N 256)], trans = 0},
{fin = [(N 181),(N 256)], trans = 0},
{fin = [(N 256)], trans = 90},
{fin = [(N 203)], trans = 0},
{fin = [(N 239),(N 256)], trans = 0},
{fin = [(N 256)], trans = 93},
{fin = [], trans = 94},
{fin = [(N 211)], trans = 0},
{fin = [(N 236),(N 256)], trans = 0},
{fin = [(N 233),(N 256)], trans = 0},
{fin = [(N 243),(N 256)], trans = 0},
{fin = [(N 249),(N 256)], trans = 99},
{fin = [(N 249)], trans = 0},
{fin = [(N 258),(N 262),(N 269)], trans = 0},
{fin = [(N 267),(N 269)], trans = 102},
{fin = [(N 267)], trans = 0},
{fin = [(N 253)], trans = 0},
{fin = [(N 251),(N 253)], trans = 0},
{fin = [(N 7),(N 253)], trans = 62},
{fin = [(N 4),(N 253)], trans = 64},
{fin = [(N 159)], trans = 0},
{fin = [(N 155),(N 159)], trans = 109},
{fin = [(N 155)], trans = 109},
{fin = [(N 151),(N 159)], trans = 111},
{fin = [(N 151)], trans = 111},
{fin = [(N 157),(N 159)], trans = 0},
{fin = [(N 7),(N 159)], trans = 62},
{fin = [(N 4),(N 159)], trans = 64}])
end
structure StartStates =
	struct
	datatype yystartstate = STARTSTATE of int

(* start state definitions *)

val AFTERDOT = STARTSTATE 11;
val COMMENT = STARTSTATE 5;
val HELDOVER = STARTSTATE 3;
val INITIAL = STARTSTATE 1;
val STRING = STARTSTATE 7;
val STRINGSKIP = STARTSTATE 9;

end
type result = UserDeclarations.lexresult
	exception LexerError (* raised if illegal leaf action tried *)
end

fun makeLexer yyinput =
let	val yygone0=1
	val yyb = ref "\n" 		(* buffer *)
	val yybl = ref 1		(*buffer length *)
	val yybufpos = ref 1		(* location of next character to use *)
	val yygone = ref yygone0	(* position in file of beginning of buffer *)
	val yydone = ref false		(* eof found yet? *)
	val yybegin = ref 1		(*Current 'start state' for lexer *)

	val YYBEGIN = fn (Internal.StartStates.STARTSTATE x) =>
		 yybegin := x

fun lex () : Internal.result =
let fun continue() = lex() in
  let fun scan (s,AcceptingLeaves : Internal.yyfinstate list list,l,i0) =
	let fun action (i,nil) = raise LexError
	| action (i,nil::l) = action (i-1,l)
	| action (i,(node::acts)::l) =
		case node of
		    Internal.N yyk => 
			(let val yytext = substring(!yyb,i0,i-i0)
			     val yypos = i0+ !yygone
			open UserDeclarations Internal.StartStates
 in (yybufpos := i; case yyk of 

			(* Application actions *)

  103 => (SCON(SCon.NumCon(ICF.Decimal,ICF.Unsigned,suffix(yytext,2)),
      yypos,yypos+size yytext))
| 115 => (SCON(SCon.NumCon(ICF.Hex,ICF.Unsigned,suffix(yytext,3)),
      yypos,yypos+size yytext))
| 13 => (TokenTable.lookup_alpha
   {characters= !yyb,initial=i0,length=i-i0,position=yypos})
| 133 => (SCON(SCon.RealCon yytext,yypos,yypos+size yytext))
| 137 => (YYBEGIN AFTERDOT;DOTHASHHASH(yypos,yypos+3))
| 140 => (YYBEGIN AFTERDOT;DOTHASH(yypos,yypos+2))
| 142 => (YYBEGIN AFTERDOT;DOT(yypos,yypos+1))
| 151 => (YYBEGIN INITIAL;TokenTable.lookup_alpha_unreserved
      {characters= !yyb,initial=i0,length=i-i0,position=yypos})
| 155 => (YYBEGIN INITIAL;TokenTable.lookup_symbolic_unreserved
      {characters= !yyb,initial=i0,length=i-i0,position=yypos})
| 157 => (YYBEGIN STRING;
    string_type:=LexState.JAVASYMBOL;
    string_start:=yypos;
    continue())
| 159 => (err(Error.error({left=yypos,right=yypos},
    "Missing identifier after ."));
    YYBEGIN INITIAL;
    ALPHA(bad_symbol,yypos,yypos)
    )
| 162 => (YYBEGIN COMMENT;
    LexState.comment_level:=1;
    LexState.comment_start:=yypos;
    continue())
| 165 => (LexState.comment_level:= !LexState.comment_level + 1;
    continue())
| 168 => (let
       val cl= !LexState.comment_level - 1
       val ()= LexState.comment_level:=cl
       val ()=if cl=0 then YYBEGIN INITIAL else ()
    in
       continue()
    end
    )
| 170 => (continue())
| 173 => (LexState.err(Error.error({left=yypos,right=yypos+2},
      "*\041 not permitted outside comments"));
(* ML-Lex bracket counts parentheses in strings! *)
    continue()
    )
| 175 => (
   (* Character and string constants.  These are dealt with by a similar
      trick to SML/NJ; we lump them together and keep a variable
      string_type which reveals what we are doing.  The only slight
      complication is that we also have to cope with Java backquoted
      symbols.  We allow double quotes to occur in Java backquoted
      symbols and back quotes to occur in string constants. 
      We allow \' in both backquoted constants and string constants.
      *)
   YYBEGIN STRING;
   string_type:=LexState.STRING;
   string_start:=yypos;
   continue())
| 178 => (YYBEGIN STRING;
   string_type:=LexState.CHAR;
   string_start:=yypos;
   continue())
| 18 => (TokenTable.lookup_symbolic
   {characters= !yyb,initial=i0,length=i-i0,position=yypos})
| 181 => (string_contents:= 0w7:: !string_contents;
   continue()
   )
| 184 => (string_contents:= 0w8:: !string_contents;
   continue()
   )
| 187 => (string_contents:= 0w9:: !string_contents;
   continue()
   )
| 190 => (string_contents:= 0w10:: !string_contents;
   continue()
   )
| 193 => (string_contents:= 0w11:: !string_contents;
   continue()
   )
| 196 => (string_contents:= 0w12:: !string_contents;
   continue()
   )
| 199 => (string_contents:= 0w13:: !string_contents;
   continue()
   )
| 203 => (string_contents:=
      (Word.fromInt(Char.ord(String.sub(yytext,2)))-0w32) 
      :: !string_contents;
   continue()
   )
| 211 => (string_contents:=
      Word.fromInt(atoi(String.substring(yytext,1,3)))
      :: !string_contents;
   continue()
   )
| 230 => (string_contents:=
      atow(String.substring(yytext,2,4))
      :: !string_contents;
   continue()
   )
| 233 => (string_contents:=
     0wx22 :: !string_contents;
   continue()
   )
| 236 => (string_contents:=
      0wx27 :: !string_contents;
    continue()
    )
| 239 => (string_contents:=
     0wx5c :: !string_contents;
   continue()
   )
| 24 => (TokenTable.lookup_tyvar
   {characters= !yyb,initial=i0,length=i-i0,position=yypos})
| 243 => (YYBEGIN STRINGSKIP;
   continue()
   )
| 249 => (YYBEGIN STRINGSKIP;
   do_newline(yypos+1);
   continue()
   )
| 251 => (YYBEGIN STRING;continue())
| 253 => (err(Error.error({left= !string_start,right=yypos},
    "Illegal \\[whitespace] escape in "^(cors())^" constant"));
    YYBEGIN STRING;
    continue())
| 256 => (let
      val ch=String.sub(yytext,1)
   in
     (err(Error.error({left= !string_start,right=yypos},
         "Illegal \\"^Char.toString ch^" escape in "^(cors())^" constant"));
      string_contents:=Word.fromInt(Char.ord ch) :: !string_contents;
      continue()
      )
   end
   )
| 258 => (if !string_type=LexState.JAVASYMBOL
    then
      (string_contents:= 0wx22:: !string_contents; continue())
    else
      (YYBEGIN INITIAL;end_string yypos)
    )
| 26 => (LPAREN(yypos,yypos+1))
| 260 => (if !string_type<>LexState.JAVASYMBOL
    then
      (string_contents:= 0wx60:: !string_contents; continue())
    else
      (YYBEGIN INITIAL;end_string yypos)
    )
| 262 => (
   (* Any other normal character (This rule has to come late so it
      doesn't take precedence over the others). *)
   string_contents:=
      Word.fromInt(Char.ord(
         String.sub(!yyb,i0)
         )) 
      :: !string_contents;
   continue()
   )
| 267 => (err(Error.error({left= !string_start,right=yypos},
    "Unterminated "^(cors())^" constant"));
    do_newline yypos;
    YYBEGIN INITIAL;
    end_string yypos)
| 269 => (err(Error.error({left=yypos,right=yypos},
    "Illegal character in string"));
    continue())
| 271 => (err(Error.error({left=yypos,right=yypos},"Illegal character"));
    continue())
| 28 => (RPAREN(yypos,yypos+1))
| 30 => (LBRACKET(yypos,yypos+1))
| 32 => (RBRACKET(yypos,yypos+1))
| 34 => (LBRACE(yypos,yypos+1))
| 36 => (RBRACE(yypos,yypos+1))
| 38 => (COMMA(yypos,yypos+1))
| 4 => (do_newline yypos;continue())
| 40 => (SEMICOLON(yypos,yypos+1))
| 44 => (DOTS(yypos,yypos+1))
| 46 => (WILD(yypos,yypos+1))
| 48 => (BAR(yypos,yypos+1))
| 55 => ((* MLJ reserved words, and also _ followed by other keywords *)

    case TokenTable.lookup_underline
       {characters= !yyb,initial=i0,length=i-i0,position=yypos} 
   of
      TokenTable.ONE tok => tok
   |  TokenTable.TWO tok => 
         (held_over:=tok;YYBEGIN HELDOVER;WILD(yypos,yypos+1))
   )
| 58 => (let 
      val ho= !held_over 
      val ()= held_over:=bad_token 
(* It should be possible to remove this statement; it is there to cause 
   trouble if the caller tries to change lexers in midstream *)
   in
      (YYBEGIN INITIAL;ho)
   end
   )
| 62 => (INTLAB(yytext,yypos,yypos+size yytext))
| 67 => (SCON(SCon.NumCon(ICF.Decimal,ICF.Signed false,yytext),
      yypos,yypos+size yytext))
| 7 => (continue())
| 73 => (SCON(SCon.NumCon(ICF.Decimal,ICF.Signed true,suffix(yytext,1)),
      yypos,yypos+size yytext))
| 84 => (SCON(SCon.NumCon(ICF.Hex,ICF.Signed false,suffix(yytext,2)),
      yypos,yypos+size yytext))
| 96 => (SCON(SCon.NumCon(ICF.Hex,ICF.Signed true,suffix(yytext,3)),
      yypos,yypos+size yytext))
| _ => raise Internal.LexerError

		) end )

	val {fin,trans} = Vector.sub(Internal.tab, s)
	val NewAcceptingLeaves = fin::AcceptingLeaves
	in if l = !yybl then
	     if trans = #trans(Vector.sub(Internal.tab,0))
	       then action(l,NewAcceptingLeaves
) else	    let val newchars= if !yydone then "" else yyinput 1024
	    in if (size newchars)=0
		  then (yydone := true;
		        if (l=i0) then UserDeclarations.eof ()
		                  else action(l,NewAcceptingLeaves))
		  else (if i0=l then yyb := newchars
		     else yyb := substring(!yyb,i0,l-i0)^newchars;
		     yygone := !yygone+i0;
		     yybl := size (!yyb);
		     scan (s,AcceptingLeaves,l-i0,0))
	    end
	  else let val NewChar = Char.ord(String.sub(!yyb,l))
		val NewState = if NewChar<128 then Char.ord(String.sub(trans,NewChar)) else Char.ord(String.sub(trans,128))
		in if NewState=0 then action(l,NewAcceptingLeaves)
		else scan(NewState,NewAcceptingLeaves,l+1,i0)
	end
	end
(*
	val start= if substring(!yyb,!yybufpos-1,1)="\n"
then !yybegin+1 else !yybegin
*)
	in scan(!yybegin (* start *),nil,!yybufpos,!yybufpos)
    end
end
  in lex
  end
end
